require 'common/plugin'
require 'common/filesystem'

module DovecotPlugin
  # Code that all Dovecot plugins (Prune, Rm, Mv...) will
  # share.  That is, we implement the Plugin interface.
  include Plugin

  def initialize(cfg)
    @domain_root = cfg.mail_root
  end

  def describe_domain(domain)
    begin
      domain_path = get_domain_path(domain)
      return domain_path
    rescue NonexistentDomainError => e
      return "Doesn't exist: #{e.to_s}"
    end
  end

  def describe_account(account)
    begin
      account_path = get_account_path(account)
      return account_path
    rescue NonexistentAccountError => e
      return "Doesn't exist: #{e.to_s}"
    end
  end


  protected;

  def get_domain_path(domain)
    # Return the filesystem path for the given domain.
    # That is, the directory where its mail is stored.
    # Only works if the domain directory exists!
    domain_path = File.join(@domain_root, domain)

    if File.directory?(domain_path)
      return domain_path
    else
      raise NonexistentDomainError.new(domain)
    end
  end


  def get_account_path(account)
    # Return the filesystem path of this account's mailbox.
    # Only works if the account exists!
    if not account.include?('@')
      msg = "#{account}: Accounts must contain an '@' symbol."
      raise InvalidAccountError.new(msg)
    end

    account_parts = account.split('@')
    user_part = account_parts[0]
    domain_part = account_parts[1]

    begin
      domain_path = get_domain_path(domain_part)
    rescue NonexistentDomainError
      raise NonexistentAccountError.new(account)
    end

    account_path = File.join(domain_path, user_part)

    if File.directory?(account_path)
      return account_path
    else
      raise NonexistentAccountError.new(account)
    end
  end


  def list_domains()
    return Filesystem.get_subdirs(@domain_root)
  end

  def list_domains_users(domains)
    accounts = []

    domains.each do |domain|
      begin
        # Throws a NonexistentDomainError if the domain's path
        # doesn't exist on the filesystem. In this case, we want
        # to report zero accounts.
        domain_path = get_domain_path(domain)
        usernames = Filesystem.get_subdirs(domain_path)

        usernames.each do |username|
          accounts << "#{username}@#{domain}"
        end
      rescue NonexistentDomainError
        # Party hard.
      end
    end

    return accounts
  end


  def list_users()
    domains = list_domains()
    users = list_domains_users(domains)
    return users
  end

end
